<?php
namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DTO\Response\Error;

class Fai33000LancamentoDeDespesasDiversas extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $dataSourceName;
	protected $parameterBagColumns = array('CDFILIAL', 'DATAFINAL', 'DATAINICIAL', 'ALLDATA', 'CDDESPPREV');
	protected $dataSourceNameToSave;
	protected $dataSourceManager;

	public function __construct($crudService, $parameterBag, $dataSourceManager){
		$this->dataSourceManager = $dataSourceManager;
		$this->crudService 		 = $crudService;
		$this->parameterBag 	 = $parameterBag;
	}

	public function find(DTO\Request\Filter $request, DTO\Response $response){
		$this->dataSourceName = 'fai33000_despprevfili_despprev';
		$this->parameterBagColumns = array('CDFILIAL', 'DATAFINAL', 'DATAINICIAL', 'ALLDATA', 'CDDESPPREV');
		parent::find($request, $response);
	}

	public function save(DTO\Request $request, DTO\Response $response){
		$row = $request->getDataSet()->getRows()[0];
		if($row['__is_new']){
			$this->dataSourceNameToSave = 'despprevfili';
		} else {
			$this->dataSourceNameToSave = 'fai33000_despprevfili_despprev';
		}

		parent::save($request, $response);
	}

	public function delete(DTO\Request $request, DTO\Response $response){
		$this->dataSourceName = 'despprevfili';
		parent::delete($request, $response);
	}

	public function autoOperation($request, $response){
		try{
			$row = $request->getRow();
			$despesasCadastradas = $this->findDespprev($row);
			$dataSave = $this->mountDataSave($row, $despesasCadastradas);
			$this->saveDespprev($dataSave);
		} catch (\Exception $e) {
			$response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
		}
	}

	private function saveDespprev(&$dataSave){
		$currentTime = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
		foreach($dataSave as &$row){
			$conditions = $this->mountConditions($row);
			$dataDespprev = $this->crudService->getFilteredDataSet('despprevfili', $conditions)->getRows();
			if(!empty($dataDespprev)){
				$dataDespprev = $dataDespprev[0];
				$row['VRPRDESPFILI'] = $row['VRPRDESPFILI'] + $dataDespprev['VRPRDESPFILI'];
				$row['VRREDESPFILI'] = $row['VRREDESPFILI'] + $dataDespprev['VRREDESPFILI'];
				$row['IDATIVO'] = $dataDespprev['IDATIVO'];
				$row['DTINCLUSAO'] = $dataDespprev['DTINCLUSAO'];
				$row['NRORGINCLUSAO'] = $dataDespprev['NRORGINCLUSAO'];
				$row['CDOPERINCLUSAO'] = $dataDespprev['CDOPERINCLUSAO'];
				$row['__is_new'] = false;
			} else {
				$row['DTINCLUSAO'] = $currentTime;
				$row['NRORGINCLUSAO'] = $this->crudService->getEnvironment()->getNrOrg();
				$row['CDOPERINCLUSAO'] = $this->crudService->getEnvironment()->getCdOperador();
				$row['IDATIVO'] = 'S';
			}
			$row['DTULTATU'] = $currentTime;
			$row['NRORGULTATU'] = $this->crudService->getEnvironment()->getNrOrg();
			$row['CDOPERULTATU'] = $this->crudService->getEnvironment()->getCdOperador();
		}
		$this->crudService->save('despprevfili',$dataSave);
	}

	private function mountConditions($row){
		$conditions = array(
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $row['CDFILIAL']
			),
			array(
				'columnName' => 'CDDESPPREV',
				'operator' => '=',
				'value' => $row['CDDESPPREV']
			),
			array(
				'columnName' => 'DTPRDESPFILI',
				'operator' => '=',
				'value' => $row['DTPRDESPFILI']
			),
			array(
				'columnName' => 'NRORG',
				'operator' => '=',
				'value' => $row['NRORG']
			)
		);

		return $conditions;
	}

	private function findDespprev($row){
		$conditions = array(
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $this->crudService->getEnvironment()->getCdFilial()
			),
			array(
				"columnName" => "CDDESPPREV",
				"operator" => "=",
				"value" => $row['CDDESPPREV']
			),
			array(
				"columnName" => "DATAINICIAL",
				"operator" => "=",
				"value" => $row['DATAINICIAL']
			),
			array(
				"columnName" => "DATAFINAL",
				"operator" => "=",
				"value" => $row['DATAFINAL']
			),
			array(
				"columnName" => "ALLDATA",
				"operator" => "=",
				"value" => ""
			),
			array(
				"columnName" => "NRORG",
				"operator" => "=",
				"value" => $this->crudService->getEnvironment()->getNrOrg()
			)
		);
		$filterCriteria = new FilterCriteria('fai33000_despprevfili_despprev', $conditions);
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
		return $dataSet;
	}

	private function mountDataSave($row, $despesasCadastradas){
		$dataSave = array();

		if($row['OPERATION'] == 2) {
			$function = 'mountReleaseDate';
		} else {
			$function = 'mountFirstDayDate';
		}
		foreach($despesasCadastradas as $despesa){
			$novaDataLancamento = call_user_func_array(array($this, $function), array($row['DTLANCAMENTO'], $despesa['DTPRDESPFILI']));
			$novaDespesa = array(
				'CDDESPPREV' => $despesa['CDDESPPREV'],
				'CDFILIAL' => $despesa['CDFILIAL'],
				'CDPRODUTO' => $despesa['CDPRODUTO'],
				'DTPRDESPFILI' => $novaDataLancamento,
				'NRORG' => $despesa['NRORG'],
				'VRPRDESPFILI' => $despesa['VRPRDESPFILI'],
				'VRREDESPFILI' => $despesa['VRREDESPFILI'],
				'__is_new' => true
			);
			$this->comparaDataSet($dataSave, $novaDespesa);
		}
		return $dataSave;
	}

	private function comparaDataSet(&$dataSave, $newRow){
		$change = false;
		foreach($dataSave as &$row){
			if(($row['CDDESPPREV'] == $newRow['CDDESPPREV']) && ($row['CDFILIAL'] == $newRow['CDFILIAL']) &&  ($row['CDPRODUTO'] == $newRow['CDPRODUTO']) && ($row['DTPRDESPFILI'] == $newRow['DTPRDESPFILI'])){
				$row['VRPRDESPFILI'] = $row['VRPRDESPFILI'] + $newRow['VRPRDESPFILI'];
				$row['VRREDESPFILI'] = $row['VRREDESPFILI'] + $newRow['VRREDESPFILI'];
				return true;
			}
		}
		array_push($dataSave, $newRow);
	}

	private function mountReleaseDate($dtLancamento, $oldDate){
		$day =  substr($oldDate, 0, 2);
		$lastDayOfMonth = substr($dtLancamento, 0, 2);
		$newDate;
		if(intval($day) > intval($lastDayOfMonth)){
			$newDate = $dtLancamento;
		} else {
			$newDate = $day.substr($dtLancamento, 2, 10);
		}
		return $newDate;
	}

	private function mountFirstDayDate($dtLnacamento, $oldDate){
		return $dtLnacamento;
	}

	public function findDistinctDespprev($request, $response){
		$this->dataSourceName = 'fai33000_distinct_despprevfili_despprev';
		$this->parameterBagColumns = array('CDFILIAL', 'DATAFINAL', 'DATAINICIAL');
		parent::find($request, $response);
	}
}